@@ -1,10 +1,10 @@
 from ilines import ilines
-from itertools import chain
 import csv
 import codecs
 import chardet
 
 from messytables.core import RowSet, TableSet, Cell
+import messytables
 
 
 class UTF8Recoder:
@@ -12,6 +12,13 @@ class UTF8Recoder:
     Iterator that reads an encoded stream and reencodes the input to UTF-8
     """
     def __init__(self, f, encoding):
+        sample = f.readline()
+        if not encoding:
+            results = chardet.detect(sample)
+            encoding = results['encoding']
+            if not encoding:
+                raise Exception('Could not determine encoding')
+        f.seek(0)
         self.reader = codecs.getreader(encoding)(f, 'ignore')
 
         # The reader only skips a BOM if the encoding isn't explicit about its
@@ -54,28 +61,26 @@ class CSVTableSet(TableSet):
     """ A CSV table set. Since CSV is always just a single table,
     this is just a pass-through for the row set. """
 
-    def __init__(self, fileobj, delimiter=None, name=None, encoding=None):
-        self.fileobj = fileobj
+    def __init__(self, fileobj, delimiter=None, quotechar=None, name=None, encoding=None, window=None):
+        self.fileobj = messytables.seekable_stream(fileobj)
         self.name = name or 'table'
-        self.delimiter = delimiter or ','
-        if encoding:
-            self.encoding = encoding
-        else:
-            buf = fileobj.read(100)
-            results = chardet.detect(buf)
-            self.encoding = results['encoding']
-            fileobj.seek(0)
+        self.delimiter = delimiter
+        self.quotechar = quotechar
+        self.encoding = encoding
+        self.window = window
 
     @classmethod
-    def from_fileobj(cls, fileobj, delimiter=',', name=None):
-        return cls(fileobj, delimiter=delimiter, name=name)
+    def from_fileobj(cls, fileobj, delimiter=None, quotechar=None, name=None, encoding=None, window=None):
+        return cls(fileobj, delimiter=delimiter, quotechar=quotechar, name=name, encoding=encoding, window=window)
 
     @property
     def tables(self):
         """ Return the actual CSV table. """
         return [CSVRowSet(self.name, self.fileobj,
                           delimiter=self.delimiter,
-                          encoding=self.encoding)]
+                          quotechar=self.quotechar,
+                          encoding=self.encoding,
+                          window=self.window)]
 
 
 class CSVRowSet(RowSet):
@@ -84,15 +89,18 @@ class CSVRowSet(RowSet):
     a sample is read and cached so you can run analysis on the
     fragment. """
 
-    def __init__(self, name, fileobj, delimiter=None,
-                 encoding='utf-8', window=1000):
+    def __init__(self, name, fileobj, delimiter=None, quotechar=None,
+                 encoding='utf-8', window=None):
         self.name = name
-        self.fileobj = UTF8Recoder(fileobj, encoding)
+        seekable_fileobj = messytables.seekable_stream(fileobj)
+        self.fileobj = UTF8Recoder(seekable_fileobj, encoding)
         self.lines = ilines(self.fileobj)
         self._sample = []
-        self.delimiter = delimiter or ','
+        self.delimiter = delimiter
+        self.quotechar = quotechar
+        self.window = window or 1000
         try:
-            for i in xrange(window):
+            for i in xrange(self.window):
                 self._sample.append(self.lines.next())
         except StopIteration:
             pass
@@ -111,6 +119,16 @@ class CSVRowSet(RowSet):
         except csv.Error:
             return csv.excel
 
+    @property
+    def _overrides(self):
+        # some variables in the dialect can be overriden
+        d = {}
+        if self.delimiter:
+            d['delimiter'] = self.delimiter
+        if self.quotechar:
+            d['quotechar'] = self.quotechar
+        return d
+
     def raw(self, sample=False):
         def rows():
             for line in self._sample:
@@ -119,7 +137,8 @@ class CSVRowSet(RowSet):
                 for line in self.lines:
                     yield line
         try:
-            for row in csv.reader(rows(), delimiter=self.delimiter, dialect=self._dialect):
+            for row in csv.reader(rows(),
+                    dialect=self._dialect, **self._overrides):
                 yield [Cell(to_unicode_or_bust(c)) for c in row]
         except csv.Error, err:
             if 'newline inside string' in unicode(err) and sample:
