@@ -18,15 +18,11 @@ class CellType(object):
 
     guessing_weight = 1
 
-    def __init__(self):
-        if locale.getlocale() == (None, None):
-            locale.setlocale(locale.LC_ALL, '')
-
     @classmethod
     def test(cls, value):
         """ Test if the value is of the given type. The
         default implementation calls ``cast`` and checks if
-        that throws an exception. """
+        that throws an exception. Returns the casted value or None"""
         try:
             ins = cls()
             ins.cast(value)
